var Analytics = BaseAnalytics.extend({
  init: function(logging, templator, datastore) {
    this._super(logging, templator, datastore);
  },

  _installFrame: function(doc, url) {
    var browser = doc.createElementNS(XUL_NS_URI, 'browser');
    browser.setAttribute('type', 'content');
    browser.setAttribute('disablehistory', false);
    browser.setAttribute('src', url);
    doc.documentElement.appendChild(browser);
    if (browser.webNavigation) {
      browser.webNavigation.sessionHistory =
         Components.classes["@mozilla.org/browser/shistory;1"].
              createInstance(Components.interfaces.nsISHistory);
    }
    return browser;
  },

  _install_platform_listeners: function() {
    var newAddonManager = true, self = this;
    var versioncomp = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
                .getService(Components.interfaces.nsIVersionComparator);

    // the methods for the recorder
    var recorder = {
      uninstall: function() {
        self.trackEvent('Operations', 'Uninstall', '2.0.19');
      },
      disable: function() {
        self.trackEvent('Operations', 'Disable', '2.0.19');
      },
      cancel: function() {
        self.trackEvent('Operations', 'Cancel', '2.0.19');
      },
      updowngrade: function() {
        self.trackEvent('Operations', 'UpDownGrade', '2.0.19');
      },
      upgrade: function(newVersion) {
        self.trackEvent('Operations', 'UpgradeFrom', '2.0.19');
        self.trackEvent('Operations', 'UpgradeTo', newVersion);
      },
      downgrade: function(newVersion) {
        self.trackEvent('Operations', 'DowngradeFrom', '2.0.19');
        self.trackEvent('Operations', 'DowngradeTo', newVersion);
      }
    };

    try {
      // to get around parser errors of doing Components.utils.import
      var importfunc = Components.utils['import'];
      importfunc("resource://gre/modules/AddonManager.jsm");
    } catch (e) {
      newAddonManager = false;
    }

    if (!newAddonManager) {
      // for Firefox < 4
      var obssvc = Components.classes['@mozilla.org/observer-service;1'].
                      getService(Components.interfaces.nsIObserverService);
      var oldobs = {
        observe: function(subject, topic, data) {
          var item = subject.QueryInterface(Components.interfaces.nsIUpdateItem);
          if (item.id != '{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}') return;

          // can't get the new version of the addon very easily
          if (data == 'item-upgraded') recorder.updowngrade();
          else if (data == 'item-uninstalled') recorder.uninstall();
          else if (data == 'item-cancel-action') recorder.cancel();
          else if (data == 'item-disabled') recorder.disable();
        }
      };

      obssvc.addObserver(oldobs, 'em-action-requested', false);

    } else {
      // for Firefox >= 4
      var newListener = {
        onEnabling: function(addon, needsRestart) {},
        onEnabled: function(addon) {},
        onDisabling: function(addon, needsRestart) {
          if (addon.id != '{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}') return;
          recorder.disable();
        },
        onDisabled: function(addon) {},
        onInstalling: function(addon, needsRestart) {
          if (addon.id != '{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}') return;
          var v = versioncomp.compare('2.0.19', addon.version);
          if (v < 0) recorder.upgrade(addon.version);
          else if (v > 0) recorder.downgrade(addon.version);
        },
        onInstalled: function(addon) {},
        onUninstalling: function(addon, needsRestart) {
          if (addon.id != '{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}') return;
          recorder.uninstall();
        },
        onUninstalled: function(addon) {},
        onOperationCanceled: function(addon) {
          if (addon.id != '{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}') return;
          recorder.cancel();
        },
        onPropertyChanged: function(addon, properties) { }
      };

      AddonManager.addAddonListener(newListener);
    }
  }
});