var BaseAnalytics;

(function() {
var VERSION = 3;
var HTTP_ANALYTICS_URL = 'http://static.getforecastfox.com/extension/${platform}/${version}/analytics.html';
var MAX_CACHED_MESSAGES = 50;

BaseAnalytics = Class.extend({
  _KEY: 'analytics-',
  _message_cache: [],
	init: function(logging, templator, datastore) {
		this._logger = logging.getLogger('analytics');
		this._datastore = datastore;
		var doc = templator._document, self = this,
        url = templator.render(HTTP_ANALYTICS_URL, {
          version: VERSION,
          platform: Forecastfox.environment
    });

    this._node = this._installFrame(doc, url);
    this._install_platform_listeners();

    setTimeout(function() {
      self._do_ping();
      setInterval(function () { self._do_ping(); }, 6 * 60 * 60 * 1000);
    }, 1000 * 30);

    this._logger.debug('init complete');
	},

	_pinged: function(update_ping_time /* optional*/) {
	  var last_ping = 0;

	  if (update_ping_time) {
	    last_ping = (new Date()).toDateString();
	    this._datastore.set(this._KEY + 'ping-time', last_ping);
	    return new Date(last_ping);
	  }

	  last_ping = this._datastore.get(this._KEY + 'ping-time');
	  return new Date(last_ping ? last_ping : 0);
	},

	_do_ping: function() {
	  var last_ping = this._pinged(),
	        current = new Date((new Date()).toDateString());

	  if (current > last_ping) {
	    this._logger.debug('pinging');
	    this.trackEvent('Operations', 'ServicesLoad', '2.0.19');
	    this._pinged(true);
	  } else
	    this._logger.debug('waiting to ping');

    if (!this._datastore.get(this._KEY + 'installed')) {
      this._datastore.set(this._KEY + 'installed', (new Date()).valueOf()+'');
      this.trackEvent('Operations', 'Installed', '2.0.19');
    }
	},

	_installFrame: function(doc, url) {
	  this._logger.error('installFrame unimplemented');
	},

	trackPageview: function(page) {
		this._logger.debug('tracking pageview: ' + page);
		this._dispatch_message(['_trackPageview', page]);
  },

  trackEvent: function(category, action, optional_label, optional_value) {
    if (optional_value && optional_value == 'HOUR')
      optional_value = (new Date()).getHours();

    if (optional_label !== undefined && optional_value === undefined)
      optional_value = 1;

    this._logger.debug('tracking events: ' +
          [category, action, optional_label, optional_value].join(', '));
    this._dispatch_message(['_trackEvent', category, action, optional_label, optional_value]);
  },

  _dispatch_message: function(message) {
    this._send_message(message);

    // cache the analytics messages, for debugging purposes.
    this._message_cache.push(message);
    if (this._message_cache.length >= MAX_CACHED_MESSAGES)
      this._message_cache.shift();
  },

  _send_message: function(message) {
    try {
      var doc = this._node.contentWindow.document;
      var t = new Templator(logging, doc);
      var e = doc.createEvent('Event');
      e.initEvent('ext-to-web-event', true, false);

      t.node('#ext-to-web').append('<span>'+JSON.stringify(message)+'</span>');
      t.node('#ext-to-web')[0].dispatchEvent(e);
    } catch (e) {
      this._logger.debug('ERROR: trying to send message: ' + e);
    }
  }
});

})();
